local AceGUI = LibStub("AceGUI-3.0")

local zonePick, bossPick, datePick, groupPick, btnBench;

local editPlayer, benchPlayer;

function MorthalinRaidTools:ConstructHistorySettings()
	self.ui.hSettingsFrame = AceGUI:Create("Frame");
	self.ui.hSettingsFrame:SetTitle("History Settings");
	self.ui.hSettingsFrame:SetStatusText("");
	self.ui.hSettingsFrame:SetLayout("Flow");
	self.ui.hSettingsFrame:SetWidth(650);
	self.ui.hSettingsFrame:SetHeight(170);
	self.ui.hSettingsFrame:EnableResize(false);
	self.ui.hSettingsFrame.frame:EnableMouse(true);
	self.ui.hSettingsFrame.frame:SetUserPlaced(true);
	
	local historyPickGroup = AceGUI:Create("SimpleGroup");
	historyPickGroup:SetFullWidth(true);
	historyPickGroup:SetLayout("Flow");
	self.ui.hSettingsFrame:AddChild(historyPickGroup);
	
	zonePick = AceGUI:Create("Dropdown");
	zonePick:SetLabel("Zone");
	zonePick:SetWidth(150);
	zonePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:GetDates(); end);
	historyPickGroup:AddChild(zonePick);
	
	datePick = AceGUI:Create("Dropdown");
	datePick:SetLabel("Date");
	datePick:SetWidth(150);
	datePick:SetDisabled(true);
	datePick:SetCallback("OnValueChanged", function() MorthalinRaidTools:GetBosses(); end);
	historyPickGroup:AddChild(datePick);
	
	bossPick = AceGUI:Create("Dropdown");
	bossPick:SetLabel("Boss");
	bossPick:SetWidth(150);
	bossPick:SetDisabled(true);
	bossPick:SetCallback("OnValueChanged", function() if(bossPick:GetValue() ~= nil) then groupPick:SetDisabled(false); else groupPick:SetDisabled(true); end end);
	historyPickGroup:AddChild(bossPick);
	
	groupPick = AceGUI:Create("Dropdown");
	groupPick:SetLabel("Group");
	groupPick:SetWidth(150);
	groupPick:SetDisabled(true);
	groupPick:SetCallback("OnValueChanged", function() if(groupPick:GetValue() ~= nil and benchPlayer ~= nil and string.len(benchPlayer) ~= 0) then btnBench:SetDisabled(false); else btnBench:SetDisabled(true); end end);
	MorthalinRaidTools:GetGroups();
	historyPickGroup:AddChild(groupPick);
	
	editPlayer = AceGUI:Create("EditBox");
	editPlayer:SetLabel("Insert player:");
	editPlayer:DisableButton(true);
	editPlayer:SetCallback("OnTextChanged", function(widget, event, text)
		benchPlayer = text;
		if(string.len(text) ~= 0) then
			self:ConstructWhisperer(editPlayer, benchPlayer, 2);
			if(groupPick:GetValue() ~= nil) then 
				btnBench:SetDisabled(false); 
			end
		else
			self:HideWhisperer();
			btnBench:SetDisabled(true);
		end
	end)
	editPlayer:SetCallback("OnEnterPressed", function(widget, event, text) MorthalinRaidTools:AddHistoryBench(benchPlayer); end)
	self.ui.hSettingsFrame:AddChild(editPlayer);
	
	btnBench = AceGUI:Create("Button");
	btnBench:SetText("Add to Raid");
	btnBench:SetCallback("OnClick", function() MorthalinRaidTools:AddHistoryBench(benchPlayer); end)
	btnBench:SetDisabled(true);
	self.ui.hSettingsFrame:AddChild(btnBench);
	
	btnCreateRaid = AceGUI:Create("Button");
	btnCreateRaid:SetText("Create new raid");
	btnCreateRaid:SetCallback("OnClick", function() MorthalinRaidTools:ConstructHistoryCreation() end)
	self.ui.hSettingsFrame:AddChild(btnCreateRaid);
	
	MorthalinRaidTools:GetZones();
	self.ui.hSettingsFrame:Hide();
end

function MorthalinRaidTools:ShowHistorySettings()
	if(self.ui.hSettingsFrame:IsVisible() == false) then
		self.ui.hSettingsFrame:Show();
	end
end

function MorthalinRaidTools:HideHistorySettings()
	if(self.ui.hSettingsFrame:IsVisible()) then
		self.ui.hSettingsFrame:Hide();
	end
end

function MorthalinRaidTools:GetGroups()
	groupPick:SetList({});
	for i = 1, 8 do
		groupPick:AddItem(i, "Group " .. i);
	end
	groupPick:AddItem(9, "Bench");
end

function MorthalinRaidTools:GetZones()
	zonePick:SetList({});
	local zones = {};
	for i = 1, #MorthalinRaidTools.db.global.history do
		if(zones[MorthalinRaidTools.db.global.history[i].zone] == nil) then
			zones[MorthalinRaidTools.db.global.history[i].zone] = {zone = MorthalinRaidTools.db.global.history[i].zone};
		end
	end
	
	zones = self:ConvertStringIndex(zones);
	local function sortZones(a, b)
		return a.index < b.index;
	end
	table.sort(zones, sortZones);
	
	for i = 1, #zones do
		zonePick:AddItem(i, zones[i].index);
	end
	zonePick:AddItem(#zones + 1, nil);
	zonePick:SetValue(nil);
	datePick:SetValue(nil);
	datePick:SetDisabled(true);
	bossPick:SetValue(nil);
	bossPick:SetDisabled(true);
	groupPick:SetValue(nil);
	groupPick:SetDisabled(true);
	btnBench:SetDisabled(true);
end

function MorthalinRaidTools:GetDates()
	if(zonePick.text:GetText() ~= nil) then
		datePick:SetList({});
		local dates = {};
		for i = 1, #MorthalinRaidTools.db.global.history do
			local dateTemp;
			if (date("%H",MorthalinRaidTools.db.global.history[i].killDate) == "00" or date("%H",MorthalinRaidTools.db.global.history[i].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(MorthalinRaidTools.db.global.history[i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",MorthalinRaidTools.db.global.history[i].killDate)
			end
			if(MorthalinRaidTools.db.global.history[i].zone == zonePick.text:GetText() and dates[dateTemp] == nil) then
				dates[dateTemp] = {killDate = dateTemp};
			end
		end
		
		dates = self:ConvertStringIndex(dates);
		local function sortDates(a, b)
			local dayA, monthA, yearA = a.index:match('^(%d%d).(%d%d).(%d%d%d%d)$');
			local dateAInt = time{year=yearA, month=monthA, day=dayA};
			local dayB, monthB, yearB = b.index:match('^(%d%d).(%d%d).(%d%d%d%d)$');
			local dateBInt = time{year=yearB, month=monthB, day=dayB};
			return dateAInt > dateBInt;
		end
		table.sort(dates, sortDates);
		
		for i = 1, #dates do
			datePick:AddItem(i, dates[i].index);
		end
		datePick:SetDisabled(false);
		datePick:SetValue(nil);
		bossPick:SetValue(nil);
		bossPick:SetDisabled(true);
		groupPick:SetValue(nil);
		groupPick:SetDisabled(true);
		btnBench:SetDisabled(true);
	else
		datePick:SetDisabled(true);
		datePick:SetValue(nil);
		bossPick:SetValue(nil);
		bossPick:SetDisabled(true);
	groupPick:SetValue(nil);
	groupPick:SetDisabled(true);
		btnBench:SetDisabled(true);
	end
end

function MorthalinRaidTools:GetBosses()
	if(zonePick.text:GetText() ~= nil) then
		bossPick:SetList({});
		local bosses = {};
		local instances = {};
		local instanceCount = 0;
		for i = 1, #MorthalinRaidTools.db.global.history do
			local dateTemp;
			if (date("%H",MorthalinRaidTools.db.global.history[i].killDate) == "00" or date("%H",MorthalinRaidTools.db.global.history[i].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(MorthalinRaidTools.db.global.history[i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",MorthalinRaidTools.db.global.history[i].killDate)
			end
			if(MorthalinRaidTools.db.global.history[i].zone == zonePick.text:GetText() and dateTemp == datePick.text:GetText() and instances[MorthalinRaidTools.db.global.history[i].instanceID] == nil) then
				instanceCount = instanceCount + 1
				instances[MorthalinRaidTools.db.global.history[i].instanceID] = instanceCount;
			end
			if(MorthalinRaidTools.db.global.history[i].zone == zonePick.text:GetText() and dateTemp == datePick.text:GetText() and bosses[MorthalinRaidTools.db.global.history[i].bossName] == nil) then
				bosses[MorthalinRaidTools.db.global.history[i].bossName .. " " .. instances[MorthalinRaidTools.db.global.history[i].instanceID]] = {bossName = MorthalinRaidTools.db.global.history[i].bossName .. " " .. instances[MorthalinRaidTools.db.global.history[i].instanceID]};
			end
		end
		
		bosses = self:ConvertStringIndex(bosses);
		local function sortBosses(a, b)
			return a.index < b.index;
		end
		table.sort(bosses, sortBosses);
		
		for i = 1, #bosses do
			bossPick:AddItem(i, bosses[i].index);
		end
		bossPick:AddItem(#bosses + 1, "All");
		bossPick:SetDisabled(false);
		bossPick:SetValue(nil);
		btnBench:SetDisabled(true);
		groupPick:SetValue(nil);
		groupPick:SetDisabled(true);
	else
		bossPick:SetDisabled(true);
		bossPick:SetValue(nil);
		btnBench:SetDisabled(true);
		groupPick:SetValue(nil);
		groupPick:SetDisabled(true);
	end
end

function MorthalinRaidTools:AddHistoryBench(name)
	local temp;
	if(string.len(name) > 1) then
		temp = string.upper(string.utf8sub(name, 1, 1))..string.lower(string.utf8sub(name, 2));
	elseif(string.len(name) == 1) then
		temp = string.upper(string.utf8sub(name, 1, 1));
	end
	local playerTemp = nil;
	local group = groupPick:GetValue();
	if(group == 9) then
		group = 0;
	end
	local numTotalMembers = GetNumGuildMembers();
	for i = 1, numTotalMembers do
		local name, rank, rankIndex, level, class, zone, note, officernote, online, status = GetGuildRosterInfo(i);
		if(temp == string.sub(name, 1, string.len(temp))) then
			playerTemp = {
				name = string.sub(name, 1, (-1) * (string.len(GetRealmName()) + 2)),
				class = class,
				color = MorthalinRaidTools:GetClassColor(class),
				group = group
			}
		end
	end
			
	local bossName = bossPick.text:GetText();
	local bossNum = 1;
	local killDates = {}
	
	if(MorthalinRaidTools.db.global.encounterList[bossPick.text:GetText()] == nil) then
		local parts = {strsplit(" ", bossPick.text:GetText())};
		bossNum = parts[#parts];
		bossName = string.sub(bossPick.text:GetText(), 1, (-2 - string.len(tostring(bossNum))));
	end
	
	local foundTimes = 1;
	if(playerTemp ~= nil) then
		for i = 1, #MorthalinRaidTools.db.global.history do
			local dateTemp;
			if (date("%H",MorthalinRaidTools.db.global.history[i].killDate) == "00" or date("%H",MorthalinRaidTools.db.global.history[i].killDate) == "01") then
				dateTemp = date("%d.%m.%Y",(MorthalinRaidTools.db.global.history[i].killDate - 7200))
			else
				dateTemp = date("%d.%m.%Y",MorthalinRaidTools.db.global.history[i].killDate)
			end
			
			if(MorthalinRaidTools.db.global.history[i].zone == zonePick.text:GetText() and dateTemp == datePick.text:GetText() and (bossName == "All" or bossName == MorthalinRaidTools.db.global.history[i].bossName) and bossNum == tostring(foundTimes)) then
				local found = false;
				for j = 1, #MorthalinRaidTools.db.global.history[i].members do
					if(MorthalinRaidTools.db.global.history[i].members[j].name == playerTemp.name) then
						found = true;
					end
				end
				
				if(found) then
					MorthalinRaidTools:Print("|cffff0000Player " .. playerTemp.name .. " was in raid on kill " .. MorthalinRaidTools.db.global.history[i].bossName .. ".");
				else
					table.insert(MorthalinRaidTools.db.global.history[i].members, playerTemp);
					MorthalinRaidTools:Print("Player " .. playerTemp.name .. " was added to kill " .. MorthalinRaidTools.db.global.history[i].bossName .. ".");
					MorthalinRaidTools:BroadcastHistoryAdd(MorthalinRaidTools.db.global.history[i]);
					MorthalinRaidTools:UpdateHistory();
				end
			elseif(MorthalinRaidTools.db.global.history[i].zone == zonePick.text:GetText() and dateTemp == datePick.text:GetText() and (bossPick.text:GetText() == "All" or bossName == MorthalinRaidTools.db.global.history[i].bossName) and bossNum ~= tostring(foundTimes)) then
				foundTimes = foundTimes + 1;
			end
		end
	else
		MorthalinRaidTools:Print("|cffff0000Player " .. name .. " is not in guild or not exist.");
	end
	
	editPlayer:SetText("");
	self:HideWhisperer();
	btnBench:SetDisabled(true);
end